﻿using System;
using System.IO;
using System.Xml;
using System.Data;

namespace POSPrintSvr.DAL
{
    /// <summary>
    /// 配置文件管理类
    /// </summary>
    partial class PosPrintSvrSet
    {
        /// <summary>
        /// 读取设置
        /// </summary>
        public void ReadSeting()
        {
            if (!File.Exists(AppDomain.CurrentDomain.BaseDirectory + "PosPrintSvrSet.xml"))
            {
                InitDbConnectStrCfgItem();
                InitIsWriteLogCfgItem();
                InitMaxQuqueNumCfgItem();
                InitSleepTimeLenCfgItem();
                SaveSeting();
            }
            else
            {
                this.Clear();
                this.ReadXml(AppDomain.CurrentDomain.BaseDirectory + "PosPrintSvrSet.xml");
            }
        }
        /// <summary>
        /// 保存设置
        /// </summary>
        public void SaveSeting()
        {
            if (File.Exists(AppDomain.CurrentDomain.BaseDirectory + "PosPrintSvrSet.xml"))
            {
                File.Delete(AppDomain.CurrentDomain.BaseDirectory + "PosPrintSvrSet.xml");
            }
            WriteXml(AppDomain.CurrentDomain.BaseDirectory + "PosPrintSvrSet.xml");
        }

        /// <summary>
        /// 初始化数据库连接参数
        /// </summary>
        public void InitDbConnectStrCfgItem()
        {
            DataRow dr = SvrSetting.NewRow();
            dr[1] = "DbConnectStr";
            dr[2] = "数据库连接字符串";
            dr[3] = @"Data Source=(LOCAL);Initial Catalog=MenuPrintSvrDB;Integrated Security=True";
            dr[4] = "";
            SvrSetting.Rows.Add(dr);
        }

        /// <summary>
        /// 获取数据库连接字符串
        /// </summary>
        /// <returns></returns>
        public string GetDbConnStr()
        {
            DataView dv = SvrSetting.DefaultView;
            dv.RowFilter = "CfgCode='DbConnectStr'";
            if ((dv != null) && (dv.Count > 0))
            {
                return dv[0]["CfgValue"].ToString();
            }
            else
                return string.Empty;
        }

        /// <summary>
        /// 设置数据库连接字符串
        /// </summary>
        /// <returns></returns>
        public void SetDbConnStr(string connStr)
        {
            foreach (DataRow dr in SvrSetting.Rows)
            {
                if (dr["CfgCode"].ToString() == "DbConnectStr")
                {
                    dr["CfgValue"] = connStr;
                    break;
                }
            }           
        }


        /// <summary>
        /// 初始化数据库连接参数
        /// </summary>
        public void InitIsWriteLogCfgItem()
        {
            DataRow dr = SvrSetting.NewRow();
            dr[1] = "IsWriteLog";
            dr[2] = "是否输出日志信息";
            dr[3] = "Y";
            dr[4] = "";
            SvrSetting.Rows.Add(dr);
        }

        public string GetIsWriteLog()
        {
            DataView dv = SvrSetting.DefaultView;
            dv.RowFilter = "CfgCode='IsWriteLog'";
            if ((dv != null) && (dv.Count > 0))
            {
                return dv[0]["CfgValue"].ToString();
            }
            else
                return string.Empty;
        }

        public void SetIsWriteLog(string yesNo)
        {
            foreach (DataRow dr in SvrSetting.Rows)
            {
                if (dr["CfgCode"].ToString() == "IsWriteLog")
                {
                    dr["CfgValue"] = yesNo;
                    break;
                }
            }        
        }

        /// <summary>
        /// 初始化数据库连接参数
        /// </summary>
        public void InitMaxQuqueNumCfgItem()
        {
            DataRow dr = SvrSetting.NewRow();
            dr[1] = "MaxQuqueNum";
            dr[2] = "队列最大任务数";
            dr[3] = "100";
            dr[4] = "";
            SvrSetting.Rows.Add(dr);
        }

        public string GetMaxQuqueNum()
        {
            DataView dv = SvrSetting.DefaultView;
            dv.RowFilter = "CfgCode='MaxQuqueNum'";
            if ((dv != null) && (dv.Count > 0))
            {
                return dv[0]["CfgValue"].ToString();
            }
            else
                return string.Empty;
        }

       
        public void SetMaxQuqueNum(string num)
        {
            foreach (DataRow dr in SvrSetting.Rows)
            {
                if (dr["CfgCode"].ToString() == "MaxQuqueNum")
                {
                    dr["CfgValue"] = num;
                    break;
                }
            }           
        }
        
        public void InitSleepTimeLenCfgItem()
        {
            DataRow dr = SvrSetting.NewRow();
            dr[1] = "SleepTimeLen";
            dr[2] = "扫描线程休眠时长(豪秒)";
            dr[3] = "1000";
            dr[4] = "";
            SvrSetting.Rows.Add(dr);
        }

        public string GetSleepTimeLen()
        {
            ReadSeting();
            DataView dv = SvrSetting.DefaultView;
            dv.RowFilter = "CfgCode='SleepTimeLen'";
            if ((dv != null) && (dv.Count > 0))
            {
                return dv[0]["CfgValue"].ToString();
            }
            else
                return string.Empty;
        }

        public void SetSleepTimeLen(string num)
        {
            foreach (DataRow dr in SvrSetting.Rows)
            {
                if (dr["CfgCode"].ToString() == "SleepTimeLen")
                {
                    dr["CfgValue"] = num;
                    break;
                }
            }           
        }

        partial class SvrSettingDataTable
        {

        }
    }
}
