﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.ServiceProcess;
using System.Text;
using System.Threading;
using System.Collections;

namespace PosPrintService
{
    /// <summary>
    /// 后厨分单打印服务
    /// *使用了队列和多线程技术，保证及时打印和并发控制*
    /// </summary>
    public partial class PosPrintService : ServiceBase
    {       
        //打印队列
        private Queue _PrintTaskQueue = new Queue();
        //加载队列的线程
        private Thread _LoadThread;
        //执行打印的线程
        private Thread _PrintThread;

        #region 服务的参数
        /// <summary>
        /// 一次搜索的最大任务数,默认10个
        /// </summary>
        private int MaxSerchTaskNum = 10;
        /// <summary>
        /// 搜索间隔,默认1秒
        /// </summary>
        private int SerchThreadInterval = 1000;
        /// <summary>
        /// 打印任务队列最大任务数,默认100
        /// </summary>
        private int _PrintQueueMaxTaskNum = 100;
        /// <summary>
        /// 是否输出日志记录,默认输出
        /// </summary>
        private bool IsWriteDebugLog = true;
        #endregion

        POSPrintSvr.BLL.MenuPrintRequest menuPrintBll = new POSPrintSvr.BLL.MenuPrintRequest();
        POSPrintSvr.BLL.CookRooms cookRoomBll = new POSPrintSvr.BLL.CookRooms();

        public PosPrintService()
        {
            InitializeComponent();           
            
        }
        //启动
        protected override void OnStart(string[] args)       
        {
            // TODO: 在此处添加代码以启动服务。
            ////搜索服务管理是否运行，如果没有运行则开启
            //bool isExists = false;
            //System.Diagnostics.Process[] Processes = System.Diagnostics.Process.GetProcesses();
            //foreach (System.Diagnostics.Process process in Processes)
            //{
            //    if (process.ProcessName == "PrintServiceMgr")
            //    {
            //        isExists = true;
            //        break;
            //    }
            //}
            //if (!isExists)
            //{
            //    System.Diagnostics.ProcessStartInfo psi = new System.Diagnostics.ProcessStartInfo();
            //    psi.FileName = AppDomain.CurrentDomain.BaseDirectory + "PrintServiceMgr.exe";
            //    psi.Arguments = " -Auto";
            //    psi.Verb = "runas";
            //    psi.WindowStyle = System.Diagnostics.ProcessWindowStyle.Minimized;
            //    System.Diagnostics.Process.Start(psi);
            //    Thread.Sleep(2000);
            //}
            BeginSvrThread();
            if (IsWriteDebugLog)
                LogManager.WriteLog("服务日志","PosPrintService启动成功！" + DateTime.Now.ToLongTimeString());
        }
        //停止
        protected override void OnStop()
        {
            // TODO: 在此处添加代码以执行停止服务所需的关闭操作。
            if (_LoadThread != null)
            {
                _LoadThread.Abort();
                _LoadThread = null;
            }
            if (IsWriteDebugLog)
                LogManager.WriteLog("服务日志", "PosPrintService停止！" + DateTime.Now.ToLongTimeString());
        }
        //暂停
        protected override void OnPause()
        {
            if (_LoadThread != null)
            {
                _LoadThread.Suspend();
            }
            base.OnPause();
        }
        //继续
        protected override void OnContinue()
        {
            if (_LoadThread != null)
            {
                if (_LoadThread.ThreadState == System.Threading.ThreadState.Suspended
                    || _LoadThread.ThreadState == System.Threading.ThreadState.SuspendRequested)
                {
                    _LoadThread.Resume();
                }
            }
            base.OnContinue();           
        }

        /// <summary>
        /// 开始服务的主线程
        /// </summary>
        private void BeginSvrThread()
        {
            if (_LoadThread == null)
            {
                _LoadThread = new Thread(DetectionPrintRequest);
                _LoadThread.Start();
            }
            if (_LoadThread != null)
            {
                if (_LoadThread.ThreadState == System.Threading.ThreadState.Suspended
                    || _LoadThread.ThreadState == System.Threading.ThreadState.SuspendRequested)
                {
                    _LoadThread.Resume();
                }
            }           
        }

        /// <summary>
        /// 检测打印请求，并放入队列
        /// </summary>
        private void DetectionPrintRequest()
        {
            while (true)
            {
                if (_PrintTaskQueue.Count < _PrintQueueMaxTaskNum)
                {
                    try
                    {
                        DataSet ds_loadMenus = menuPrintBll.GetList(MaxSerchTaskNum, "PrintFlag='N'", "SendTime");
                        if (ds_loadMenus != null)
                        {
                            if (ds_loadMenus.Tables.Count > 0)
                            {
                                DataTable dt_loadMenus = ds_loadMenus.Tables[0];
                                foreach (DataRow dr in dt_loadMenus.Rows)
                                {
                                    _PrintTaskQueue.Enqueue(dr["KeyId"].ToString());
                                }
                            }
                        }
                    }
                    catch (Exception ex)
                    {
                        LogManager.WriteLog("Error", ex.Message);
                    }
                    StartPrintThread();
                }                                
                Thread.Sleep(SerchThreadInterval);
            }
        }

        /// <summary>
        /// 启动执行打印的线程
        /// </summary>
        private void StartPrintThread()
        {
            _PrintThread = new Thread(ExecuteOperate_PrintBill);
            _PrintThread.Start();
        }

        /// <summary>
        /// 开始分单打印。
        /// </summary>
        private void ExecuteOperate_PrintBill()
        {
            while (_PrintTaskQueue.Count > 0)
            {
                object obj_KeyId = _PrintTaskQueue.Dequeue();
                ExecuteMenuPrint(int.Parse(obj_KeyId.ToString()));
            }
        }

       
        /// <summary>
        /// 打印分单
        /// </summary>
        /// <param name="KeyId">流水号</param>
        void ExecuteMenuPrint(int KeyId)
        {
            BeiYangOPOS opos = new BeiYangOPOS();
            POSPrintSvr.Model.MenuPrintRequest aPrintRequest = menuPrintBll.GetModel(KeyId);
            POSPrintSvr.Model.CookRooms cookRoomModel=cookRoomBll.GetModel(aPrintRequest.CookRoomNo);
           
            if (aPrintRequest != null)
            {           
                //打印的IP一定要预先设置好
                bool b = opos.OpenNetPort(cookRoomModel.PrinterIPAddress.Trim());
                if (!b)
                {
                    LogManager.WriteLog("Error",
                        string.Format("初始化'{0}'的打印机参数失败。请检测打印机配置。",
                                      cookRoomModel.CookRoomName));
                }
                Byte res = new Byte();
                int ret = BeiYangOPOS.POS_NETQueryStatus(cookRoomModel.PrinterIPAddress, out res);
                StringBuilder sb = new StringBuilder();
                #region 检测打印机状态
                if ((res & 0x10) == 0x10)
                {
                    sb.AppendLine("打印机出错！");
                }
                if ((res & 0x02) == 0x02)
                {
                    sb.AppendLine("打印机脱机！");
                }
                if ((res & 0x04) == 0x04)
                {
                    sb.AppendLine("上盖打开！");
                }
                if ((res & 0x20) == 0x20)
                {
                    sb.AppendLine("切刀出错！");
                }
                if ((res & 0x40) == 0x40)
                {
                    sb.AppendLine("纸将尽！");
                }
                if ((res & 0x80) == 0x80)
                {
                    sb.AppendLine("缺纸！");
                }
                #endregion
                if (sb.Length > 0)
                {
                    LogManager.WriteLog("Error",
                               string.Format("'{0}'的打印机处于非正常状态：{1}。请检测打印机配置。",
                                          cookRoomModel.CookRoomName, sb.ToString()));
                    return;
                }
                else
                {
                    #region 执行指令打印
                    uint width = 1;
                    if (cookRoomModel.PrintWidth == 58)
                        width = 1;
                    if (cookRoomModel.PrintWidth == 76)
                        width = 1;
                    if (cookRoomModel.PrintWidth == 80)
                        width = 2;
                    BeiYangOPOS.POS_SetRightSpacing(0);
                    BeiYangOPOS.POS_SetLineSpacing(100);
                    BeiYangOPOS.POS_S_TextOut("" + aPrintRequest.OrderType + "菜单 【" + aPrintRequest.AccountNo + "】",
                                              0, width, 4, opos.POS_FONT_TYPE_STANDARD, opos.POS_FONT_STYLE_NORMAL);
                    BeiYangOPOS.POS_FeedLine();
                    BeiYangOPOS.POS_S_TextOut("-----------------------", 0, width, 2, opos.POS_FONT_TYPE_STANDARD, opos.POS_FONT_STYLE_NORMAL);
                    BeiYangOPOS.POS_FeedLine();
                    BeiYangOPOS.POS_S_TextOut("厨房:" + cookRoomModel.CookRoomName + "  送单人:" + aPrintRequest.MenuOper, 0, width, 2, opos.POS_FONT_TYPE_STANDARD, opos.POS_FONT_STYLE_NORMAL);
                    BeiYangOPOS.POS_FeedLine();
                    BeiYangOPOS.POS_S_TextOut("桌台:" + aPrintRequest.TableNo + @"\" + aPrintRequest.TableName + "  菜类：" + aPrintRequest.MenuTypeName, 0, width, 2, opos.POS_FONT_TYPE_STANDARD, opos.POS_FONT_STYLE_NORMAL);
                    BeiYangOPOS.POS_FeedLine();
                    BeiYangOPOS.POS_S_TextOut("-----------------------", 0, width, 2, opos.POS_FONT_TYPE_STANDARD, opos.POS_FONT_STYLE_NORMAL);
                    BeiYangOPOS.POS_FeedLine();
                    BeiYangOPOS.POS_S_TextOut(aPrintRequest.MenuNum.ToString() + "  " + aPrintRequest.Unit + "  " + aPrintRequest.MenuChName, 0, width, 2, opos.POS_FONT_TYPE_STANDARD, opos.POS_FONT_STYLE_NORMAL);
                    BeiYangOPOS.POS_FeedLine();
                    if (aPrintRequest.OrderType.Trim() == "退")
                    {
                        BeiYangOPOS.POS_S_TextOut(aPrintRequest.SendOrBackReason, 0, width, 2, opos.POS_FONT_TYPE_STANDARD, opos.POS_FONT_STYLE_NORMAL);
                    }
                    else
                    {
                        BeiYangOPOS.POS_S_TextOut(aPrintRequest.CookTasteRemark, 0, width, 2, opos.POS_FONT_TYPE_STANDARD, opos.POS_FONT_STYLE_NORMAL);
                    }
                    BeiYangOPOS.POS_FeedLine();
                    BeiYangOPOS.POS_S_TextOut("-----------------------", 0, width, 2, opos.POS_FONT_TYPE_STANDARD, opos.POS_FONT_STYLE_NORMAL);
                    BeiYangOPOS.POS_FeedLine();
                    BeiYangOPOS.POS_S_TextOut(aPrintRequest.SendTime.ToString(), 0, width, 2, opos.POS_FONT_TYPE_STANDARD, opos.POS_FONT_STYLE_NORMAL);
                    BeiYangOPOS.POS_FeedLine();
                    BeiYangOPOS.POS_FeedLine();
                    BeiYangOPOS.POS_FeedLine();
                    BeiYangOPOS.POS_CutPaper(0, 200);
                    #endregion
                    //修改标识为已打印                   
                    aPrintRequest.PrintFlag = "Y";
                    object obj_CurrentSystemDateTime = POSPrintSvr.DAL.DbOperate.dbOperate.dbTools.ExecuteScalar("select getdate()");
                    if (obj_CurrentSystemDateTime != null)
                        aPrintRequest.PrintTime = DateTime.Parse(obj_CurrentSystemDateTime.ToString());
                    menuPrintBll.Update(aPrintRequest);
                    if (IsWriteDebugLog)
                        LogManager.WriteLog("PrintLog", string.Format("打印了{0}。", aPrintRequest.MenuChName + " 流水号：" + aPrintRequest.KeyID.ToString()) + DateTime.Now.ToLongTimeString());
                }
                opos.ClosePrinterPort();       
                
            }            
        }
    }
}
